"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var selectors_1 = require("../inc/selectors");
exports.resolveProp = function (target, props) {
    return typeof target === 'function' ? target(props) : target;
};
var poseDefault = function (pose, prop, defaultValue, resolveProps) {
    return pose && pose[prop] !== undefined
        ? exports.resolveProp(pose[prop], resolveProps)
        : defaultValue;
};
var startChildAnimations = function (children, next, pose, props) {
    var animations = [];
    var delay = poseDefault(pose, 'delayChildren', 0, props);
    var stagger = poseDefault(pose, 'staggerChildren', 0, props);
    var staggerDirection = poseDefault(pose, 'staggerDirection', 1, props);
    var maxStaggerDuration = (children.size - 1) * stagger;
    var generateStaggerDuration = staggerDirection === 1
        ? function (i) { return i * stagger; }
        : function (i) { return maxStaggerDuration - i * stagger; };
    Array.from(children).forEach(function (child, i) {
        animations.push(child.set(next, __assign({}, props, { delay: delay + generateStaggerDuration(i) })));
    });
    return animations;
};
var createPoseSetter = function (setterProps) { return function (next, nextProps) {
    if (nextProps === void 0) { nextProps = {}; }
    var state = setterProps.state, poses = setterProps.poses, startAction = setterProps.startAction, stopAction = setterProps.stopAction, getInstantTransition = setterProps.getInstantTransition, addActionDelay = setterProps.addActionDelay, getTransitionProps = setterProps.getTransitionProps, resolveTarget = setterProps.resolveTarget, transformPose = setterProps.transformPose;
    var children = state.children, values = state.values, props = state.props, activeActions = state.activeActions, activePoses = state.activePoses;
    var _a = nextProps.delay, delay = _a === void 0 ? 0 : _a;
    var hasChildren = children.size;
    var nextPose = poses[next];
    var baseTransitionProps = __assign({}, props, nextProps);
    var getChildAnimations = function () {
        return hasChildren
            ? startChildAnimations(children, next, nextPose, baseTransitionProps)
            : [];
    };
    var getParentAnimations = function () {
        if (!nextPose)
            return [];
        if (transformPose)
            nextPose = transformPose(nextPose, next);
        var preTransition = nextPose.preTransition, getTransition = nextPose.transition;
        if (preTransition)
            nextPose.preTransition();
        return Object.keys(selectors_1.getPoseValues(nextPose)).map(function (key) {
            return new Promise(function (complete) {
                var value = values.get(key);
                var transitionProps = __assign({}, baseTransitionProps, { key: key,
                    value: value });
                var target = resolveTarget(value, exports.resolveProp(nextPose[key], transitionProps));
                if (activeActions.has(key))
                    stopAction(activeActions.get(key));
                var resolveTransitionProps = __assign({ to: target }, transitionProps, getTransitionProps(value, target, transitionProps));
                var transition = getTransition(resolveTransitionProps);
                if (transition === false)
                    transition = getInstantTransition(value, resolveTransitionProps);
                var poseDelay = exports.resolveProp(nextPose.delay, transitionProps);
                if (delay || poseDelay) {
                    transition = addActionDelay(delay || poseDelay, transition);
                }
                activeActions.set(key, startAction(transition, complete));
                activePoses.set(key, next);
            });
        });
    };
    if (nextPose && hasChildren) {
        if (exports.resolveProp(nextPose.beforeChildren, baseTransitionProps)) {
            return Promise.all(getParentAnimations()).then(function () {
                return Promise.all(getChildAnimations());
            });
        }
        else if (exports.resolveProp(nextPose.afterChildren, baseTransitionProps)) {
            return Promise.all(getChildAnimations()).then(function () {
                return Promise.all(getParentAnimations());
            });
        }
    }
    return Promise.all(getParentAnimations().concat(getChildAnimations()));
}; };
exports.default = createPoseSetter;
//# sourceMappingURL=setter.js.map