"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var setter_1 = require("./factories/setter");
var values_1 = require("./factories/values");
var transitions_1 = require("./factories/transitions");
var transition_composers_1 = require("./inc/transition-composers");
exports.eachValue = transition_composers_1.eachValue;
exports.fromPose = transition_composers_1.fromPose;
var selectors_1 = require("./inc/selectors");
var poseFactory = function (_a) {
    var getDefaultProps = _a.getDefaultProps, defaultTransitions = _a.defaultTransitions, bindOnChange = _a.bindOnChange, startAction = _a.startAction, stopAction = _a.stopAction, readValue = _a.readValue, resolveTarget = _a.resolveTarget, createValue = _a.createValue, getInstantTransition = _a.getInstantTransition, getTransitionProps = _a.getTransitionProps, addActionDelay = _a.addActionDelay, selectValueToRead = _a.selectValueToRead, extendAPI = _a.extendAPI;
    return function (config) {
        var parentValues = config.parentValues, ancestorValues = config.ancestorValues;
        if (parentValues)
            ancestorValues.unshift({ values: parentValues });
        var activeActions = new Map();
        var activePoses = new Map();
        var children = new Set();
        var poses = transitions_1.default(selectors_1.selectPoses(config), defaultTransitions);
        var props = config.props || {};
        if (getDefaultProps)
            props = __assign({}, props, getDefaultProps(config));
        var passive = config.passive, userSetValues = config.values, initialPose = config.initialPose;
        var values = values_1.default({
            poses: poses,
            passive: passive,
            ancestorValues: ancestorValues,
            readValue: readValue,
            createValue: createValue,
            userSetValues: userSetValues,
            initialPose: initialPose,
            props: props
        });
        var state = {
            activeActions: activeActions,
            activePoses: activePoses,
            children: children,
            props: props,
            values: values
        };
        var onChange = config.onChange;
        if (onChange)
            Object.keys(onChange).forEach(bindOnChange(values, onChange));
        var set = setter_1.default({
            state: state,
            poses: poses,
            getInstantTransition: getInstantTransition,
            getTransitionProps: getTransitionProps,
            startAction: startAction,
            stopAction: stopAction,
            resolveTarget: resolveTarget,
            addActionDelay: addActionDelay
        });
        var api = {
            set: set,
            get: function (valueName) {
                return valueName
                    ? selectValueToRead(values.get(valueName))
                    : selectors_1.selectAllValues(values, selectValueToRead);
            },
            has: function (poseName) { return !!poses[poseName]; },
            setProps: function (newProps) {
                props = __assign({}, props, newProps);
            },
            _addChild: function (childConfig, factory) {
                var child = factory(__assign({ initialPose: initialPose }, childConfig, { ancestorValues: [{ label: config.label, values: values }].concat(ancestorValues) }));
                children.add(child);
                return child;
            },
            removeChild: function (child) { return children.delete(child); },
            clearChildren: function () {
                children.forEach(function (child) { return child.destroy(); });
                children.clear();
            },
            destroy: function () {
                activeActions.forEach(stopAction);
                children.forEach(function (child) { return child.destroy(); });
            }
        };
        return extendAPI(api, state);
    };
};
exports.default = poseFactory;
//# sourceMappingURL=index.js.map