export declare type Props = {
    [key: string]: any;
};
export declare type NumberPropFactory = (props: Props) => number;
export declare type BooleanPropFactory = (props: Props) => boolean;
export declare type StaggerDirectionPropFactory = (props: Props) => 1 | -1;
export declare type TransitionFactory<A> = (props: Props) => A | false;
export declare type Pose<A> = {
    transition?: TransitionFactory<A>;
    delay?: number | NumberPropFactory;
    delayChildren?: number | NumberPropFactory;
    staggerChildren?: number | NumberPropFactory;
    staggerDirection?: 1 | -1 | StaggerDirectionPropFactory;
    beforeChildren?: boolean | BooleanPropFactory;
    afterChildren?: boolean | BooleanPropFactory;
    preTransform?: () => any;
    [key: string]: any;
};
export declare type ValueMap<V> = Map<string, V>;
export declare type AncestorValue<V> = {
    label?: string;
    values: ValueMap<V>;
};
export declare type AncestorValueList<V> = Array<AncestorValue<V>>;
export declare type Transformer = (v: any) => any;
export declare type PassiveValueMap = {
    [key: string]: [string, any, boolean | string | void];
};
export declare type OnChangeCallbacks = {
    [key: string]: (v: any) => any;
};
export declare type ActiveActions<A> = Map<string, A>;
export declare type ActivePoses = Map<string, string>;
export declare type ChildPosers<V, A, P> = Set<Poser<V, A, P>>;
export declare type PoseMap<A> = {
    [key: string]: Pose<A>;
};
export declare type PoserFactory<V, A, P> = (config: PoserConfig<V>) => Poser<V, A, P>;
export interface Poser<V, A, P> {
    set: (next: string, props?: Props) => Promise<any>;
    get: (key?: string) => any;
    has: (key: string) => boolean;
    setProps: (props: Props) => void;
    destroy: () => void;
    _addChild: (config: PoserConfig<V>, factory: PoserFactory<V, A, P>) => Poser<V, A, P>;
    removeChild: (child: Poser<V, A, P>) => void;
    clearChildren: () => void;
}
export declare type PoserState<V, A, P> = {
    activeActions: ActiveActions<A>;
    activePoses: ActivePoses;
    children: ChildPosers<V, A, P>;
    values: ValueMap<V>;
    props: Props;
};
export declare type PoserConfig<V> = {
    [key: string]: any;
    label?: string;
    props?: Props;
    values?: {
        [key: string]: V;
    };
    parentValues?: ValueMap<V>;
    ancestorValues?: AncestorValueList<V>;
    onChange?: OnChangeCallbacks;
    passive?: PassiveValueMap;
    initialPose?: string | string[];
};
export declare type ReadValue<V> = (value: V) => any;
export declare type ResolveTarget<V> = (value: V, target: any) => any;
export declare type CreateValueProps = any;
export declare type CreateValue<V> = (init: any, key: string, props?: CreateValueProps) => V;
export declare type StartAction<A> = (action: A, complete: Function) => A;
export declare type StopAction<A> = (action: A) => any;
export declare type GetInstantTransition<V, A> = (value: V, props: Props) => A;
export declare type AddTransitionDelay<A> = (delay: number, transition: A) => A;
export declare type ExtendAPI<V, A, P> = (api: Poser<V, A, P>, state: PoserState<V, A, P>) => Poser<V, A, P>;
export declare type GetTransitionProps<V> = (value: V, target: number, props: Props) => Props;
export declare type SelectValueToRead<V> = (value: V) => any;
export declare type TransformPose<A> = (pose: Pose<A>, key: string) => Pose<A>;
export declare type PoseFactoryConfig<V, A, P> = {
    getDefaultProps?: (config: PoserConfig<V>) => Props;
    defaultTransitions?: Map<string, TransitionFactory<A>>;
    bindOnChange: (values: ValueMap<V>, onChange: OnChangeCallbacks) => (key: string) => any;
    readValue: ReadValue<V>;
    createValue: CreateValue<V>;
    resolveTarget: ResolveTarget<V>;
    getTransitionProps: GetTransitionProps<V>;
    startAction: StartAction<A>;
    stopAction: StopAction<A>;
    getInstantTransition: GetInstantTransition<V, A>;
    addActionDelay: AddTransitionDelay<A>;
    selectValueToRead: SelectValueToRead<V>;
    extendAPI: ExtendAPI<V, A, P>;
    transformPose?: TransformPose<A>;
};
